Option Explicit On 
Option Strict On

Public Class CCustomerID

    Private mstrCustomerID As String
    Private mfValid As Boolean
    Public RequiredLength As Integer = 5

    Public Event BeforeUpdate(ByVal pstrCustomerID As String, ByRef pfCancel As Boolean)

#Region "Inner Classes"
    Public Class InvalidCustomerIDException
        Inherits System.ApplicationException

        Public Sub New(ByVal pstrID As String)
            MyBase.New("The customer ID specified, " & pstrID & ", is not valid")
        End Sub

    End Class
#End Region
#Region "Constructors"

    Public Sub New(ByVal pID As String)
        mfValid = Validate(pID)
        If mfValid Then
            mstrCustomerID = pID
        Else
            Throw New InvalidCustomerIDException(pID)
        End If
    End Sub

    Public Sub New(ByVal pID As CCustomerID)
        ' ByVal and ByRef both pass object pointers, so if you
        ' want to create a new CustomerID instance based on an
        ' existing instance, you need to copy the values of the
        ' base datatypes to a new instance.
        mstrCustomerID = pID.CustomerID
    End Sub

#End Region

#Region "Methods"

    Private Function Validate(ByVal pstrID As String) As Boolean
        ' Strip out any leading or trailing spaces.
        pstrID = pstrID.Trim

        ' A CustomerID must have five characters.
        If pstrID.Length = Me.RequiredLength Then
            Return True
        Else
            Return False
        End If
    End Function

    Public Function Exists() As Boolean

        Dim cnn As New System.Data.OleDb.OleDbConnection(BuildCnnStr("(local)", "Northwind"))

        Dim strSQL As String = "SELECT COUNT(*) FROM Customers WHERE CustomerID = '" & mstrCustomerID & "'"

        Dim cmd As New System.Data.OleDb.OleDbCommand(strSQL, cnn)

        Dim f As Boolean

        cnn.Open()
        f = CBool(cmd.ExecuteScalar())
        cnn.Close()
        Return f

    End Function

#End Region

#Region "Properties"
    Public Property CustomerID() As String
        Get
            Return mstrCustomerID
        End Get
        Set(ByVal Value As String)

            Dim fCanceled As Boolean
            RaiseEvent BeforeUpdate(Value, fCanceled)

            If Not fCanceled Then

                mfValid = Validate(Value)
                If mfValid Then
                    mstrCustomerID = Value
                Else
                    Throw New InvalidCustomerIDException(Value)
                End If

            End If
        End Set
    End Property

    Public ReadOnly Property IsValid() As Boolean
        Get
            Return mfValid
        End Get
    End Property

#End Region

End Class
