Option Explicit On 
Option Strict On

#Region "Phone Number Datatype"
Public MustInherit Class CNumberString

    Public MustInherit Class InvalidNumberStringException
        Inherits System.ApplicationException

        Protected Sub New(ByVal pstrMessage As String)
            MyBase.New(pstrMessage)
        End Sub

    End Class

    Public Event StringValueBeforeUpdate(ByVal pstrValue As String, ByRef pCancel As Boolean)

    ' This variable stores an array of valid characters.  It is declared
    ' as Protected so that classes that inherit from this class can access
    ' this variable, but other classes cannot.
    Protected cValidChars() As String

    Protected mstrValue As String

    Public MustOverride Function ThrowException(ByVal pstrInvalidString As String) As InvalidNumberStringException

    Public Sub New()
        DefineValidChars()
    End Sub

    Public Property StringValue() As String
        Get
            Return mstrValue
        End Get
        Set(ByVal Value As String)

            Dim fCancelChange As Boolean

            If isValid(Value) Then
                RaiseEvent StringValueBeforeUpdate(Value, fCancelChange)
                If Not fCancelChange Then
                    mstrValue = Value
                End If
            Else
                Throw ThrowException(Value)
            End If

        End Set
    End Property

    Public Overridable Function IsValid(ByVal pstrNumber As String) As Boolean

        Dim i As Integer = 0
        Dim nUBound As Integer = cValidChars.GetUpperBound(0)
        Dim nLBound As Integer = cValidChars.GetLowerBound(0)

        For i = nLBound To nUBound Step 1
            pstrNumber = pstrNumber.Replace(cValidChars(i), "")
        Next
        pstrNumber = pstrNumber.Trim()
        If pstrNumber.Length > 0 Then
            Return False
        Else
            Return True
        End If
    End Function

    Protected Overridable Sub DefineValidChars()
        cValidChars = New String(9) {"1", "2", "3", "4", "5", "6", "7", "8", "9", "0"}
    End Sub

    Public Overrides Function ToString() As String
        Return mstrValue
    End Function


End Class

Public Class CPhoneNo
    Inherits CNumberString

    Public Class InvalidPhoneNumberException
        Inherits CNumberString.InvalidNumberStringException

        Public Sub New(ByVal pstrPhoneNumber As String)
            MyBase.New("The phone number specified, " & pstrPhoneNumber & ", contains invalid characters.")
        End Sub

    End Class

    Sub New(ByVal pPhoneNo As CPhoneNo)
        Me.New(pPhoneNo.StringValue)
    End Sub

    Sub New(ByVal pstrPhoneNo As String)
        MyBase.new()
        Me.StringValue = pstrPhoneNo
    End Sub

    Public Overrides Function ThrowException(ByVal pstrInvalidPhone As String) As InvalidNumberStringException
        Throw New InvalidPhoneNumberException(pstrInvalidPhone)
    End Function


    ' This is the only new change.  This sub redefines the DefineValidChars method,
    ' so when this method is called in the base class's constructor, it will
    ' call this method, instead of the original method.
    Protected Overrides Sub DefineValidChars()
        cValidChars = New String(14) {"1", "2", "3", "4", "5", "6", "7", "8", "9", "0", "(", ")", " ", ".", "-"}
    End Sub
End Class


Public Class CExtension
    Inherits CNumberString

    Public Class InvalidPhoneExtensionException
        Inherits CNumberString.InvalidNumberStringException

        Public Sub New(ByVal pstrPhoneNumber As String)
            MyBase.New("The phone extension specified, " & pstrPhoneNumber & ", contains invalid characters.")
        End Sub

    End Class

    Public Sub New(ByVal pExt As CExtension)
        Me.New(pExt.StringValue)
    End Sub

    Public Sub New(ByVal pstrExtension As String)
        MyBase.New()
        Me.StringValue = pstrExtension
    End Sub

    Public Overrides Function ThrowException(ByVal pstrInvalidExt As String) As InvalidNumberStringException
        Throw New InvalidPhoneExtensionException(pstrInvalidExt)
    End Function

End Class

Public Class CBusinessPhone
    Inherits CPhoneNo

    Private mExt As CExtension

    Public Sub New(ByVal pBusinessPhone As CBusinessPhone)
        Me.New(pBusinessPhone.StringValue, New CExtension(pBusinessPhone.Extension.StringValue))
    End Sub

    Sub New(ByVal pstrPhone As String, ByVal pExt As CExtension)
        MyBase.New(pstrPhone)
        Me.Extension = pExt
    End Sub

    Public Property Extension() As CExtension
        Get
            Return mExt
        End Get
        Set(ByVal Value As CExtension)
            mExt = Value
        End Set
    End Property

    Public Overrides Function ToString() As String
        If mExt.StringValue.Length > 0 Then
            Return Me.StringValue & " ext. " & mExt.StringValue
        Else
            Return Me.StringValue
        End If
    End Function

End Class

#End Region

Public Class CSocialSecurityNo
    Inherits CNumberString

    Public Class InvalidSSNException
        Inherits CNumberString.InvalidNumberStringException

        Sub New(ByVal pstrSSN As String)
            MyBase.New("The SSN specified, " & pstrSSN & ", is not valid.")
        End Sub

    End Class

    Public Sub New(ByVal pSSN As CSocialSecurityNo)
        Me.New(pSSN.StringValue)
    End Sub

    Public Sub New(ByVal pstrSSN As String)
        MyBase.new()
        StringValue = pstrSSN
    End Sub

    Public Overrides Function isValid(ByVal pstrSSN As String) As Boolean
        If MyBase.isValid(pstrSSN) Then
            Dim nFirstHyphen As Integer = pstrSSN.IndexOf("-")
            If nFirstHyphen = 3 Then
                If pstrSSN.IndexOf("-", nFirstHyphen + 1) = 6 Then

                    If pstrSSN.Chars(0) = "0" Then
                        Return False
                    Else
                        Return True
                    End If

                Else
                    Return False
                End If
            Else
                Return False
            End If
        End If

    End Function

    Public Overrides Function ThrowException(ByVal pstrInvalidSSN As String) As InvalidNumberStringException
        Throw New InvalidSSNException(pstrInvalidSSN)
    End Function

    Protected Overrides Sub DefineValidChars()
        cValidChars = New String(10) {"1", "2", "3", "4", "5", "6", "7", "8", "9", "0", "-"}
    End Sub


End Class

